package org.w3.directmapping.servlet

import javax.servlet.http.{HttpServlet, HttpServletRequest, HttpServletResponse} 
import scala.xml.XML
import scala.xml.dtd.{DocType, PublicID}

import org.w3.rdf._
import org.w3.rdb.RDB._
import org.w3.sql
import org.w3.rdf.turtle.TurtleModule
import org.w3.directmapping.DirectMappingModule
import java.io.File

import org.w3.rdf.jena._

object DirectMappingWebapp {

  val defaultSQL = 
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING, addr INT, FOREIGN KEY (addr) REFERENCES Addresses(ID));
INSERT INTO People (ID, fname, addr) VALUES (7, "Bob", 18);
INSERT INTO People (ID, fname, addr) VALUES (8, "Sue", NULL);
"""

  val encoding = "utf-8"

  val Style = """
    #sql { background-color: #e7e7ff; float:left; border: thin solid #888888; padding: 1em; }
    #result { background-color: #eee; float:left; border: thin solid #888888; padding: 1em; }
    .clear { clear:both; }
    input { margin-bottom: -1em; }
"""

  val scripts = Map(
    "emp_addr" ->
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING, addr INT, FOREIGN KEY (addr) REFERENCES Addresses(ID));
INSERT INTO People (ID, fname, addr) VALUES (7, "Bob", 18);
INSERT INTO People (ID, fname, addr) VALUES (8, "Sue", NULL);""",

    "hier_tabl_proto" ->
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING, addr INT,
                     FOREIGN KEY (addr) REFERENCES Addresses(ID));
INSERT INTO People (ID, fname, addr) VALUES (7, "Bob", 18);
CREATE TABLE Offices (ID INT PRIMARY KEY,
                      building INT, ofcNumber STRING,
                      FOREIGN KEY (ID) REFERENCES Addresses(ID));
INSERT INTO Offices (ID, building, ofcNumber) VALUES (18, 32, "G528");""",

    "hier_table" ->
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING, addr INT,
                     FOREIGN KEY (addr) REFERENCES Addresses(ID));
INSERT INTO People (ID, fname, addr) VALUES (7, "Bob", 18);
CREATE TABLE Offices (ID INT PRIMARY KEY,
                      building INT, ofcNumber STRING,
                      FOREIGN KEY (ID) REFERENCES Addresses(ID));
INSERT INTO Offices (ID, building, ofcNumber) VALUES (18, 32, "G528");
CREATE TABLE ExecutiveOffices (ID INT PRIMARY KEY,
                               desk STRING,
                               FOREIGN KEY (ID) REFERENCES Offices(ID));
INSERT INTO ExecutiveOffices (ID, desk) VALUES (18, "oak");""",

    "multi_key" ->
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE Department (ID INT PRIMARY KEY, name STRING, city STRING,
                         manager INT, FOREIGN KEY (manager) REFERENCES People(ID), UNIQUE (name, city));
INSERT INTO Department (ID, name, city, manager) VALUES (23, "accounting", "Cambridge", 8);
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING,
                     addr INT, FOREIGN KEY (addr) REFERENCES Addresses(ID),
                     deptName STRING, deptCity STRING, FOREIGN KEY (deptName, deptCity) REFERENCES Department(name, city));
INSERT INTO People (ID, fname, addr, deptName, deptCity) VALUES (7, "Bob", 18, "accounting", "Cambridge");
INSERT INTO People (ID, fname, addr, deptName, deptCity) VALUES (8, "Sue", NULL, NULL, NULL);""",

    "ref_no_pk" ->
"""CREATE TABLE Addresses (ID INT PRIMARY KEY, city STRING, state STRING);
INSERT INTO Addresses (ID, city, state) VALUES (18, "Cambridge", "MA");
CREATE TABLE Department (ID INT PRIMARY KEY, name STRING, city STRING, manager INT,
                         FOREIGN KEY (manager) REFERENCES People(ID),
                         UNIQUE (name, city));
INSERT INTO Department (ID, name, city, manager) VALUES (23, "accounting", "Cambridge", 8);
CREATE TABLE People (ID INT PRIMARY KEY, fname STRING, addr INT,
                     FOREIGN KEY (addr) REFERENCES Addresses(ID),
                     deptName STRING, deptCity STRING,
                     FOREIGN KEY (deptName, deptCity) REFERENCES Department(name, city));
INSERT INTO People (ID, fname, addr, deptName, deptCity) VALUES (7, "Bob", 18, "accounting", "Cambridge");
INSERT INTO People (ID, fname, addr, deptName, deptCity) VALUES (8, "Sue", NULL, NULL, NULL);
CREATE TABLE Projects (lead INT,
                       FOREIGN KEY (lead) REFERENCES People(ID),
                       name STRING, UNIQUE (lead, name), 
                       deptName STRING, deptCity STRING,
                       UNIQUE (name, deptName, deptCity),
                       FOREIGN KEY (deptName, deptCity) REFERENCES Department(name, city));
INSERT INTO Projects (lead, name, deptName, deptCity) VALUES (8, "pencil survey", "accounting", "Cambridge");
INSERT INTO Projects (lead, name, deptName, deptCity) VALUES (8, "eraser survey", "accounting", "Cambridge");
CREATE TABLE TaskAssignments (worker INT,
                              FOREIGN KEY (worker) REFERENCES People(ID),
                              project STRING, PRIMARY KEY (worker, project), 
                              deptName STRING, deptCity STRING,
                              FOREIGN KEY (worker) REFERENCES People(ID),
                              FOREIGN KEY (project, deptName, deptCity) REFERENCES Projects(name, deptName, deptCity),
                              FOREIGN KEY (deptName, deptCity) REFERENCES Department(name, city));
INSERT INTO TaskAssignments (worker, project, deptName, deptCity) VALUES (7, "pencil survey", "accounting", "Cambridge");""",

    "1table0rows" ->
"""CREATE TABLE Student (
  Name VARCHAR(50)
);""",

    "1table1row" ->
"""CREATE TABLE Student (
  Name VARCHAR(50)
);
INSERT INTO Student (Name) VALUES ("Venus");""",

    "1table2columns1row" ->
"""CREATE TABLE Student (
  ID INT,
  Name VARCHAR(50)
);
INSERT INTO Student (ID, Name) VALUES(10,"Venus");""",

    "1table3columns1row" ->
"""CREATE TABLE Student (
  ID INT,
  FirstName VARCHAR(50),
  LastName VARCHAR(50)
);
INSERT INTO Student (ID, FirstName, LastName) VALUES (10,"Venus", "Williams");""",

    "2duplicates0nulls" ->
"""CREATE TABLE IOUs (
  fname CHAR(20),
  lname CHAR(20),
  amount FLOAT);
INSERT INTO IOUs (fname, lname, amount) VALUES ("Bob", "Smith", 30);
INSERT INTO IOUs (fname, lname, amount) VALUES ("Sue", "Jones", 20);
INSERT INTO IOUs (fname, lname, amount) VALUES ("Bob", "Smith", 30);""",

    "varchar_varchar_1row" ->
"""CREATE TABLE Student_Sport(
  Student VARCHAR(50),
  Sport VARCHAR(50)
);
INSERT INTO Student_Sport (Student,Sport) VALUES ("Venus", "Tennis");""",

    "1table1compositeprimarykey3columns1row" ->
"""CREATE TABLE Student (
  ID integer,
  Name varchar(50),
  Sport varchar (50),
  PRIMARY KEY (ID,Name)
);
INSERT INTO Student (ID, Name,Sport) VALUES(10,"Venus Williams","Tennis");""",

    "1table1primarykey1column1row" ->
"""CREATE TABLE Student (
Name varchar(50) PRIMARY KEY
);
INSERT INTO Student (Name) VALUES ("Venus");""",

    "2tables1primarykey1foreingkey" ->
"""CREATE TABLE Student (
ID integer,
Name varchar(50),
Sport integer,
PRIMARY KEY (ID),
FOREIGN KEY(Sport) REFERENCES Sport(ID)
);

CREATE TABLE Sport (
ID integer,
Name varchar (50),
PRIMARY KEY (ID)
);

INSERT INTO Student (ID, Name, Sport) VALUES (10,"Venus Williams", 100);
INSERT INTO Student (ID, Name, Sport) VALUES (20,"Demi Moore", NULL);
INSERT INTO Sport (ID, Name) VALUES (100,"Tennis");""")

  def format(varr:String, sql:String):String = {
    val formattedSQL = sql.replaceAll("\n+$", "").replaceAll("\n", "\\\\n\\\\\n")
    "var " + varr + " = '" + formattedSQL + "';"
  }

  def validJSvar(varr:String) = {
    if ("^[0-9]".r.findFirstIn(varr).isDefined)
      "_" + varr
    else
      varr
  }

  val Script:String = scripts map { case (varr, sql) => format(validJSvar(varr), sql) } mkString "\n\n"

  def renderVar(varr:String) = 
    <p><input value={ varr } type="button" onclick={ "document.getElementById('sql').value = " + validJSvar(varr) + ";" } /></p>

}

import DirectMappingWebapp._

class DirectMappingWebapp extends HttpServlet with JenaModel with DirectMappingModule with TurtleModule {

  import DirectMapping._

  val SQLParser = sql.SqlParser()

  val turtleParser = new TurtleParser { }

  def jenaSerializer(g:Graph):String = {
    val m = com.hp.hpl.jena.rdf.model.ModelFactory.createModelForGraph(g.jenaGraph)
    val s = new java.io.StringWriter
    m.write(s, "N-TRIPLE")
    s.toString
  }

  override def doPost(request:HttpServletRequest, response:HttpServletResponse) = {
    val hierarchy = request.getParameter("hierarchy") == "on"
    request.getParameter("sql") match {
      case null | "" => processIndex(request, response)
      case sql       => processSQL(request, response, sql, hierarchy)
    }
  }

  override def doGet(request:HttpServletRequest, response:HttpServletResponse) = doPost(request, response)

  val xhtmlDoctype = DocType("html",
                             PublicID("-//W3C//DTD XHTML 1.0 Strict//EN",
                                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"),
                             Nil)

  def render(sql:String, result:Option[String]) =
    <html xmlns="http://www.w3.org/1999/xhtml">
      <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <title>Direct Mapping Demo</title>
      <style type="text/css" media="print, screen and (min-width: 481px)">{ Style }
      </style>
      <script type="text/javascript">{ Script }
      </script>
      </head>
      <body>
        <h1>Direct Mapping Demo</h1>
	<p>Input some DDL/SQL and see the resulting <a href="http://www.w3.org/2001/sw/rdb2rdf/directMapping/">Direct Graph</a>. This interface uses GET (vs. POST) so the length of the input SQL will be limited on servers which impose a limit on the URL length.</p>
	<p>Please send bug reports to <a href="mailto:public-rdb2rdf-comments@w3.org">public-rdb2rdf-comments@w3.org</a>.</p>
        <form method="get" action="/">
          <p><textarea rows="15" cols="80" name="sql" id="sql">{ sql }</textarea></p>
	  <div style="float:right;">
	  <div style="float:left;">
            { List("emp_addr", "hier_tabl_proto", "hier_table", "multi_key", "ref_no_pk") map renderVar }
	  </div>
	  <div style="float:left;">
            { List("1table0rows", "1table1row", "1table2columns1row", "1table3columns1row", "2duplicates0nulls", "varchar_varchar_1row", "1table1compositeprimarykey3columns1row", "1table1primarykey1column1row", "2tables1primarykey1foreingkey") map renderVar }
	  </div>
	  </div>
	  <p><input name="hierarchy" checked="checked" type="checkbox" />Detects hierarchy relation</p>
	  <p><input value="clear" type="button" onclick="document.getElementById('sql').value = '';" /></p>
	  <p><input id="submit" value="submit SQL" type="submit" /></p>
        </form>
	<div class="clear"/>
        { if (result isDefined) <pre name="result" id="result">{ result.get }</pre> }
        <hr class="clear" />
        <address>
          <a href="http://www.w3.org/People/Eric/">Eric Prud'hommeaux</a>, <a href="http://www.w3.org/People/Bertails/">Alexandre Bertails</a>, Feb 2011
        </address>
      </body>
    </html>

  def processSQL(request:HttpServletRequest, response:HttpServletResponse, sql:String, hierarchy:Boolean) {

    val result =
      try {
        val db = SQLParser.toDB(sql)
        DirectMapping.HierarchyDetection = hierarchy
        val computedGraph:Graph = directDB(db)
        jenaSerializer(computedGraph)
      } catch {
        case e => e.getMessage
      }

    val html = render(sql, Some(result))

    response.setContentType("application/xml; charset='" + encoding + "'")

    XML.write(response.getWriter, html, encoding, false, xhtmlDoctype) 

  }

  def processIndex(request:HttpServletRequest, response:HttpServletResponse) {

    val index = render(defaultSQL, None)

    response.setContentType("application/xml; charset='" + encoding + "'")

    XML.write(response.getWriter, index, encoding, false, xhtmlDoctype)

  }

}

