import sbt._

trait Common extends BasicScalaProject {
  val scalatools = "scala-tools" at "http://scala-tools.org/repo-snapshots"
  val scalatest = "org.scalatest" % "scalatest" % "1.2-for-scala-2.8.0.final-SNAPSHOT" % "test"
  override def compileOptions = super.compileOptions ++ Seq(Unchecked, Deprecation, ExplainTypes)
  override def defaultExcludes = super.defaultExcludes || "*~"
}

trait TestSuite extends BasicScalaProject {
  val scalatools = "scala-tools" at "http://scala-tools.org/repo-snapshots"
  val scalatest = "org.scalatest" % "scalatest" % "1.2-for-scala-2.8.0.final-SNAPSHOT" % "compile"
  override def compileOptions = super.compileOptions ++ Seq(Unchecked, Deprecation, ExplainTypes)
  override def defaultExcludes = super.defaultExcludes || "*~"
}

class FeDeRate(info: ProjectInfo) extends ParentProject(info) {
  self =>

  val feDeRatePath = Path.fromFile(".")

  lazy val isomorphic = project("isomorphic", "isomorphic", new Isomorphic(_))
  lazy val rdb = project("rdb", "rdb", new RDB(_))
  lazy val sql = project("sql", "sql", new SQL(_), rdb)
  lazy val rdf = project("rdf", "rdf", new RDF(_), isomorphic)
  lazy val rdfmodelTest = project("rdfmodel-test", "rdfmodel-test", new RDFModelTest(_), rdf)
  lazy val rdfdsl = project("rdfdsl", "rdfdsl", new RDFDSL(_), rdf)
  lazy val turtle = project("turtle", "turtle", new Turtle(_), rdf)
  lazy val turtleTest = project("turtle-test", "turtle-test", new TurtleTest(_), turtle)
  lazy val rdfxml = project("rdfxml", "rdfxml", new RDFXML(_), rdf)
  lazy val rdfxmlTest = project("rdfxml-test", "rdfxml-test", new RDFXMLTest(_), rdfxml)
  lazy val jena = project("jena", "jena", new Jena(_), rdf)
  lazy val jenaTest = project("jena-test", "jena-test", new JenaTest(_), jena, rdfmodelTest, directmappingTest, turtleTest, rdfxmlTest)
  lazy val sharedtestdata = project("sharedtestdata", "sharedtestdata", new SharedTestData(_), rdb, rdf, sql, turtle)
  lazy val directmapping = project("directmapping", "directmapping", new DirectMapping(_), rdb, rdf, sql, sharedtestdata)
  lazy val directmappingWebapp = project("directmapping-webapp", "directmapping-webapp", new DirectMappingWebapp(_), directmapping, jena)
  lazy val directmappingTest = project("directmapping-test", "directmapping-test", new DirectMappingTest(_), directmapping, jena)
  lazy val sparql = project("sparql", "sparql", new SPARQL(_), rdf)
  lazy val sparql2sql = project("sparql2sql", "sparql2sql", new SPARQL2SQL(_), sparql, sql)
  lazy val sparql2sparql = project("sparql2sparql", "sparql2sparql", new SPARQL2SPARQL(_), sparql)
  lazy val sparql2sparql2sql = project("sparql2sparql2sql", "sparql2sparql2sql", new SPARQL2SPARQL2SQL(_), sparql2sparql, sparql2sql)
  lazy val sparql2sqlendpoint = project("sparql2sqlendpoint", "sparql2sqlendpoint", new SPARQL2SQLEndPoint(_), sparql2sql)
  lazy val sparql2sqlendpointJar = project("sparql2sqlendpointjar", "sparql2sqlendpointjar", new SPARQL2SQLEndPointJar(_), sparql2sqlendpoint)

  class Isomorphic(info: ProjectInfo) extends DefaultProject(info) with Common

  class RDB(info: ProjectInfo) extends DefaultProject(info) with Common

  class SQL(info: ProjectInfo) extends DefaultProject(info) with Common

  class RDF(info: ProjectInfo) extends DefaultProject(info) with Common

  class RDFDSL(info: ProjectInfo) extends DefaultProject(info) with Common

  class RDFModelTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite

  class Turtle(info: ProjectInfo) extends DefaultProject(info) with Common

  class TurtleTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite

  class RDFXML(info: ProjectInfo) extends DefaultProject(info) with Common {
    val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
    val jenaIri = "com.hp.hpl.jena" % "iri" % "0.8" from "http://openjena.org/repo/com/hp/hpl/jena/iri/0.8/iri-0.8.jar"
  }

  class RDFXMLTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite

  class Jena(info: ProjectInfo) extends DefaultProject(info) with Common {
    val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
    val jenaIri = "com.hp.hpl.jena" % "iri" % "0.8" from "http://openjena.org/repo/com/hp/hpl/jena/iri/0.8/iri-0.8.jar"
  }

  class JenaTest(info: ProjectInfo) extends DefaultProject(info) with Common

  class SharedTestData(info: ProjectInfo) extends DefaultProject(info) with Common

  class DirectMapping(info: ProjectInfo) extends DefaultProject(info) with Common

  class DirectMappingWebapp(info: ProjectInfo) extends DefaultWebProject(info) with Common {
    val jettyConf = config("jetty")
    val jettyDep = "org.eclipse.jetty" % "jetty-webapp" % "7.0.2.v20100331" % "compile,jetty"
    override def jettyClasspath = managedClasspath(jettyConf)
  }

  class DirectMappingTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite

  class SPARQL(info: ProjectInfo) extends DefaultProject(info) with Common

  class SPARQL2SQL(info: ProjectInfo) extends DefaultProject(info) with Common

  class SPARQL2SPARQL(info: ProjectInfo) extends DefaultProject(info) with Common

  class SPARQL2SQLEndPoint(info: ProjectInfo) extends DefaultWebProject(info) with Common with BSBMPlugin {
    val jettyConf = config("jetty")
    /* http://repo1.maven.org/maven2 */
    val jettyDep = "org.eclipse.jetty" % "jetty-webapp" % "7.0.2.v20100331" % "compile,jetty"
    override def jettyClasspath = managedClasspath(jettyConf)
    val mysql = "mysql" % "mysql-connector-java" % "5.1.13"
    val postgresql = "postgresql" % "postgresql" % "9.0-801.jdbc4"
    val codaRepo = "Coda Hale's Repository" at "http://repo.codahale.com/"
    val fig = "com.codahale" %% "fig" % "1.0.7" withSources()
    override def webappUnmanaged = super.webappUnmanaged +++ ("src" / "main" / "resources" / "database.properties")
  }

  class SPARQL2SPARQL2SQL(info: ProjectInfo) extends DefaultProject(info) with Common

  class SPARQL2SQLEndPointJar(info: ProjectInfo) extends DefaultProject(info) with ProguardProject {

    // defines the default main class
    override def mainClass: Option[String] = Some("org.w3.sparql2sqlendpoint.SPARQL2SQLEndpoint")

    // tells Proguard to take some more JAR/WAR/.class
    override def proguardInJars =
      super.proguardInJars +++
      // we need the Scala library
      scalaLibraryPath +++
      // the Proguard plugin does not considerer the .war files so we include the generated bytecode
      (self.feDeRatePath / "sparql2sqlendpoint" / "target" / "scala_2.8.0" / "classes") +++
      // ships the default configuration
      (self.feDeRatePath / "sparql2sqlendpoint" / "src" / "main" / "resources")

    override def proguardOptions = List(
      proguardKeepAllScala,
      // automatically generates the MANIFEST.MF
      proguardKeepMain(mainClass.get),
      // keeps all the SQL drivers we can find
      "-keep public class * implements java.sql.Driver"
    )

  }

}
